using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Office = Microsoft.Office.Core;
using Excel = Microsoft.Office.Interop.Excel;
using System.Xml;
using System.Windows.Forms;
using System.Configuration;
using System.Security.Permissions;

namespace CRMDocumentation
{
    class ExcelInteractionHelper
    {
        private static Object missing = Type.Missing;
        private XmlDocument xdCustomizations;

        public ExcelInteractionHelper()
        {
        }

        public void LoadXml()
        {
            try
            {
                //load the exported customizations!
                XmlDocument _xd = new XmlDocument();

                OpenFileDialog openFileDialog1 = new OpenFileDialog();
                openFileDialog1.Filter = "XML Files|*.xml";
                openFileDialog1.CheckFileExists = true;
                openFileDialog1.CheckPathExists = true;
                openFileDialog1.Multiselect = false;
                openFileDialog1.ShowDialog();

                _xd.Load(openFileDialog1.FileName);
                xdCustomizations = _xd;

                Properties.Settings mySettings = new CRMDocumentation.Properties.Settings();
                mySettings.Entities.Add("hoi");
                mySettings.Save();
                mySettings.Reload();
                

            }
            catch (Exception ex)
            {
            }
        }

        public void DocForms(Excel.Application thisApp)
        {
            try
            {
                Excel.Worksheet mySheet;
                Excel.Range myRange;

                if (xdCustomizations != null)
                {
                    CRMDocumentation.Properties.Settings mySettings = new CRMDocumentation.Properties.Settings();
                    System.Collections.Specialized.StringCollection scEntities;
                    if (mySettings.AllEntitites)
                    {
                        scEntities = new System.Collections.Specialized.StringCollection();
                        foreach (XmlNode xnForm in xdCustomizations.SelectNodes("ImportExportXml/Entities/Entity/FormXml[../Name]"))
                        {
                            string strEntityName = xnForm.SelectSingleNode("../Name").InnerText;
                            scEntities.Add(strEntityName);
                        }
                    }
                    else
                    {
                        scEntities = mySettings.Entities;
                    }

                    ArrayList myArrayList = ArrayList.Adapter(scEntities);
                    myArrayList.Sort();

                    foreach (string strEntityName in scEntities) 
                    {

                        //get the entity
                        XmlNode xnEntity = xdCustomizations.SelectSingleNode("ImportExportXml/Entities/Entity[Name='" + strEntityName + "']");

                        if (xnEntity != null && !SheetExists(thisApp, strEntityName))
                        {

                            CreateSheet(Globals.ThisAddIn.Application, strEntityName);

                            //get the forms
                            foreach (XmlNode xnForm in xnEntity.SelectNodes("FormXml/forms/entity[form/@type]/form"))
                            {
                                string strFormType = xnForm.Attributes["type"].Value;

                                WriteFormFormHeader(thisApp, strEntityName, strFormType);

                                //get the tabs
                                foreach (XmlNode xnTab in xnForm.SelectNodes("tabs/tab"))
                                {
                                    string strTabName = xnTab.SelectSingleNode("labels/label/@description").InnerText;

                                    WriteFormTabHeader(thisApp, strTabName);

                                    //get the sections
                                    foreach (XmlNode xnSection in xnTab.SelectNodes("sections/section"))
                                    {
                                        string strSectionName = xnSection.SelectSingleNode("labels/label/@description").InnerText;
                                        string strSectionShowLabel = xnSection.SelectSingleNode("@showlabel").InnerText;

                                        WriteFormSectionHeader(thisApp, strSectionName, strSectionShowLabel);
                                        thisApp.ScreenUpdating = true;
                                        thisApp.ScreenUpdating = false;
                                        WriteFormRowHeader(thisApp);

                                        //get the rows
                                        foreach (XmlNode xnRow in xnSection.SelectNodes("rows/row"))
                                        {
                                            //get the cells
                                            foreach (XmlNode xnCell in xnRow.SelectNodes("cell[control]"))
                                            {
                                                //string strEntityName
                                                //string strFormType
                                                //string strTabName
                                                //string strSectionName
                                                //string strSectionShowLabel
                                                string strFieldName = "";
                                                string strLabelOnForm = "";
                                                string strLabelOnAdvancedFind = "";
                                                string strDescription = ""; //Description as typed in when adding a new attribute to the entity
                                                string strDisabled = "";
                                                string strType = "";
                                                string strLength = "";
                                                string strRequiredLevel = "";
                                                string strMaxLength = "";
                                                string strFormat = "";
                                                string strMinValue = "";
                                                string strMaxValue = "";
                                                string strAccuracy = "";
                                                string strDefaultValue = "";
                                                                                               
                                                if (xnCell.SelectSingleNode("control/@datafieldname") != null)
                                                    strFieldName = xnCell.SelectSingleNode("control/@datafieldname").InnerText;
                                                 if (xnCell.SelectSingleNode("labels/label/@description") != null)
                                                    strLabelOnForm = xnCell.SelectSingleNode("labels/label/@description").InnerText;
                                                if (xnCell.SelectSingleNode("control/@disabled") != null)
                                                    strDisabled = xnCell.SelectSingleNode("control/@disabled").InnerText;

                                                //get remaining info from EntityInfo
                                                foreach (XmlNode xnAttribute in xnEntity.SelectNodes("EntityInfo/entity/attributes/attribute"))
                                                {
                                                    if (xnAttribute.SelectSingleNode("@PhysicalName") != null)
                                                    {
                                                        if (xnAttribute.SelectSingleNode("@PhysicalName").InnerText.ToLower() == strFieldName.ToLower())
                                                        {
                                                            if (xnAttribute.SelectSingleNode("Type") != null)
                                                                strType = xnAttribute.SelectSingleNode("Type").InnerText;
                                                            if (xnAttribute.SelectSingleNode("Description") != null)
                                                                strDescription = xnAttribute.SelectSingleNode("Description").InnerText;
                                                            if (xnAttribute.SelectSingleNode("Length") != null)
                                                                strLength = xnAttribute.SelectSingleNode("Length").InnerText;
                                                        }
                                                    }
                                                }

                                                //get remaining info from FieldXml
                                                XmlNode xnFieldXml = xnEntity.SelectSingleNode("FieldXml/entity/fields/field[@name='" + strFieldName.ToLower() + "']");
                                                if (xnFieldXml != null)
                                                {

                                                    if (xnFieldXml.SelectSingleNode("@requiredlevel") != null)
                                                        strRequiredLevel = xnFieldXml.SelectSingleNode("@requiredlevel").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("displaynames/displayname/@description") != null)
                                                        strLabelOnAdvancedFind = xnFieldXml.SelectSingleNode("displaynames/displayname/@description").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("@maxlength") != null)
                                                        strMaxLength = xnFieldXml.SelectSingleNode("@maxlength").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("@format") != null)
                                                        strFormat = xnFieldXml.SelectSingleNode("@format").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("@minvalue") != null)
                                                        strMinValue = xnFieldXml.SelectSingleNode("@minvalue").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("@maxvalue") != null)
                                                        strMaxValue = xnFieldXml.SelectSingleNode("@maxvalue").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("@accuracy") != null)
                                                        strAccuracy = xnFieldXml.SelectSingleNode("@accuracy").InnerText;
                                                    if (xnFieldXml.SelectSingleNode("@defaultvalue") != null)
                                                        strDefaultValue = xnFieldXml.SelectSingleNode("@defaultvalue").InnerText;
                                                }

                                                
                                                myRange = thisApp.ActiveCell;
                                                myRange.Value2 = strFieldName;
                                                myRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                                                myRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;

                                                if (strRequiredLevel == "required")
                                                {
                                                    myRange.Font.Color = -16776961;
                                                }
                                                if (strRequiredLevel == "recommended")
                                                {
                                                    myRange.Font.Color = -4165632;
                                                }
                                                myRange.get_Offset(0, 1).Value2 = strLabelOnForm;
                                                myRange.get_Offset(0, 1).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                                                myRange.get_Offset(0, 1).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;

                                                myRange.get_Offset(0, 2).Value2 = strLabelOnAdvancedFind;
                                                myRange.get_Offset(0, 2).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                                                myRange.get_Offset(0, 2).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;

                                                switch (strType)
                                                {
                                                    case "nvarchar":
                                                       if (strMaxLength == "") strMaxLength = strLength;
                                                        strType = strType + " " + strFormat + " (" + strMaxLength + ")";
                                                        break;
                                                    case "int":
                                                        strType = strType + " (" + strMinValue + ", " + strMaxValue + ")";
                                                        break;
                                                    default:
                                                        break;
                                                }
                                                myRange.get_Offset(0, 3).Value2 = strType;
                                                myRange.get_Offset(0, 3).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                                                myRange.get_Offset(0, 3).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                                                myRange.get_Offset(0, 3).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                                                myRange.get_Offset(0, 3).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;
                                                myRange.get_Offset(1, 0).Activate();
                                            }
                                        }
                                    }
                                }

                            }
                            FormatColumnWidth(thisApp);
                        }
                    }
                    thisApp.ScreenUpdating = true;
                    mySheet= (Excel.Worksheet) thisApp.Worksheets[mySettings.Entities[0]];
                    mySheet.Activate();
                }
            }
            
            catch (Exception ex)
            {
                thisApp.ScreenUpdating = true;
            }
        }

        public void DocPicklists(Excel.Application thisApp)
        {
            try
            {
                Excel.Worksheet mySheet;
                Excel.Range myRange;

                XmlNode xnEntity;

                if (xdCustomizations != null)
                {
                    CRMDocumentation.Properties.Settings mySettings = new CRMDocumentation.Properties.Settings();
                    System.Collections.Specialized.StringCollection scEntities;
                    if (mySettings.AllEntitites)
                    {
                        scEntities = new System.Collections.Specialized.StringCollection();
                        foreach (XmlNode xnForm in xdCustomizations.SelectNodes("ImportExportXml/Entities/Entity/FormXml[../Name]"))
                        {
                            string strEntityName = xnForm.SelectSingleNode("../Name").InnerText;
                            scEntities.Add(strEntityName);
                        }
                    }
                    else
                    {
                        scEntities = mySettings.Entities;
                    }

                    ArrayList myArrayList = ArrayList.Adapter(scEntities);
                    myArrayList.Sort();

                    foreach (string strEntityName in scEntities)
                    {
                        thisApp.ScreenUpdating = true;
                        thisApp.ScreenUpdating = false;

                        if (!SheetExists(thisApp, strEntityName))
                        {
                            // create a sheet for this entity picklist
                            mySheet = (Excel.Worksheet)thisApp.ActiveWorkbook.Worksheets.Add(missing, thisApp.Worksheets[thisApp.Worksheets.Count], missing, missing);
                            mySheet.Name = strEntityName;
                            mySheet.Activate();

                            //get the entity
                            xnEntity = xdCustomizations.SelectSingleNode("ImportExportXml/Entities/Entity[Name='" + strEntityName + "']");

                            // activate next cell to the right for formatting 
                            
                            Excel.Range nextcell = (Excel.Range)thisApp.Cells[thisApp.ActiveCell.Row, thisApp.ActiveCell.Column + 1];
                            nextcell.Activate();
                            
                            //get the picklists
                            foreach (XmlNode xnPicklistField in xnEntity.SelectNodes("FieldXml/entity/fields/field[options]"))
                            {
                                //remember the first two cells
                                Excel.Range mergecell1 = (Excel.Range)thisApp.ActiveCell;
                                mergecell1 = mergecell1.get_Offset(0, -1);
                                Excel.Range mergecell2 = (Excel.Range)thisApp.ActiveCell;
                                Excel.Range mergecell3 = (Excel.Range)mergecell1.get_Offset(1, 0);
                                Excel.Range mergecell4 = (Excel.Range)mergecell2.get_Offset(1, 0);

                                // write the schema name
                                // write the label
                                // remember the default value
                                string strDefaultOptionValue = "";
                                if (xnPicklistField.SelectSingleNode("@defaultvalue") != null)
                                    strDefaultOptionValue = xnPicklistField.SelectSingleNode("@defaultvalue").InnerText;

                                thisApp.ActiveCell.Value2 = xnPicklistField.SelectSingleNode("@name").InnerText;
                                thisApp.ActiveCell.Font.Size = 12;
                                thisApp.ActiveCell.Font.Bold = true;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Interior.Pattern = Excel.XlPattern.xlPatternSolid;
                                thisApp.ActiveCell.Interior.PatternColorIndex = Excel.XlColorIndex.xlColorIndexAutomatic;
                                thisApp.ActiveCell.Interior.ThemeColor = Excel.XlThemeColor.xlThemeColorLight1;
                                thisApp.ActiveCell.Interior.TintAndShade = 0.599993896298105;
                                thisApp.ActiveCell.Interior.PatternTintAndShade = 0;
                                thisApp.ActiveCell.get_Offset(1, 0).Activate();

                                string strDesc = "";
                                if (xnPicklistField.SelectSingleNode("displaynames/displayname/@description") != null)
                                    strDesc = xnPicklistField.SelectSingleNode("displaynames/displayname/@description").InnerText;
                                thisApp.ActiveCell.Value2 = "(" + strDesc + ")";
                                thisApp.ActiveCell.Font.Size = 10;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].Weight = Excel.XlBorderWeight.xlThin;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].Weight = Excel.XlBorderWeight.xlThin;

                                thisApp.ActiveCell.get_Offset(1, 0).Activate();

                                foreach (XmlNode xnPickListOption in xnPicklistField.SelectNodes("options/option"))
                                ///labels/label/@description"
                                {
                                    string strThisOptionValue = "";
                                    if (xnPickListOption.SelectSingleNode("@value") != null)
                                        strThisOptionValue = xnPickListOption.SelectSingleNode("@value").InnerText;
                                    Excel.Range leftcell = (Excel.Range)thisApp.Cells[thisApp.ActiveCell.Row, thisApp.ActiveCell.Column - 1];
                                    leftcell.Value2 = strThisOptionValue;

                                    thisApp.ActiveCell.Value2 = xnPickListOption.SelectSingleNode("labels/label/@description").InnerText;
                                    if (strThisOptionValue == strDefaultOptionValue)
                                        thisApp.ActiveCell.Font.Bold = true;
                                    thisApp.ActiveCell.get_Offset(1, 0).Activate();
                                }

                                Excel.Worksheet mySheet2 = (Excel.Worksheet)thisApp.ActiveSheet;

                                // autofit
                                Excel.Range myRange3 = mySheet2.get_Range(thisApp.ActiveCell.get_Offset(0, -1), thisApp.ActiveCell);
                                myRange3.EntireColumn.AutoFit();

                                // merge the first two rows
                                Excel.Range myRange2 = (Excel.Range) thisApp.get_Range(mergecell1, mergecell2);
                                myRange2.Merge(true);
                                myRange2 = (Excel.Range)thisApp.get_Range(mergecell3, mergecell4);
                                myRange2.Merge(true);

                                // activate the next column for a new picklist
                                myRange3 = (Excel.Range)thisApp.Cells[1, thisApp.ActiveCell.Column + 2];
                                myRange3.Activate();
                            }
                            Excel.Range myRange4;
                            myRange4 = mySheet.get_Range("A1", "A1");
                            myRange4.Activate();
                        }
                    }
                    mySheet = (Excel.Worksheet)thisApp.Worksheets[mySettings.Entities[0]];
                    mySheet.Activate();
                }
            }
            catch (Exception ex)
            {
            }
            finally
            {
                thisApp.ScreenUpdating = true;
            }
        }

        public void DocScripts(Excel.Application thisApp)
        {
            Excel.Worksheet mySheet;
            
            

            if (xdCustomizations != null)
            {
                CRMDocumentation.Properties.Settings mySettings = new CRMDocumentation.Properties.Settings();
                System.Collections.Specialized.StringCollection scEntities;
                if (mySettings.AllEntitites)
                {
                    scEntities = new System.Collections.Specialized.StringCollection();
                    foreach (XmlNode xnForm in xdCustomizations.SelectNodes("ImportExportXml/Entities/Entity/FormXml[../Name]"))
                    {
                        string strEntityName = xnForm.SelectSingleNode("../Name").InnerText;
                        scEntities.Add(strEntityName);
                    }
                }
                else
                {
                    scEntities = mySettings.Entities;
                }

                ArrayList myArrayList = ArrayList.Adapter(scEntities);
                myArrayList.Sort();

                foreach (string strEntityName in scEntities)
                {
                    if (!SheetExists(thisApp, strEntityName))
                    {
                        // create a sheet for this entity
                        CreateSheet(Globals.ThisAddIn.Application, strEntityName);

                        Excel.Range myRange = thisApp.ActiveCell;
                        myRange.Value2 = "level";
                        myRange.EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                        myRange.EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                        myRange.get_Offset(0, 1).Value2 = "label";
                        myRange.get_Offset(0, 1).EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                        myRange.get_Offset(0, 1).EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                        myRange.get_Offset(0, 2).Value2 = "attribute";
                        myRange.get_Offset(0, 2).EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                        myRange.get_Offset(0, 2).EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                        myRange.get_Offset(0, 3).Value2 = "event";
                        myRange.get_Offset(0, 3).EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                        myRange.get_Offset(0, 3).EntireColumn.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;
                        myRange.get_Offset(0, 4).Value2 = "script";
                        myRange.get_Offset(1, 0).Activate();

                        //get the entity
                        XmlNode xnEntity = xdCustomizations.SelectSingleNode("ImportExportXml/Entities/Entity[Name='" + strEntityName + "']");

                        //get the forms
                        foreach (XmlNode xnForm in xnEntity.SelectNodes("FormXml/forms/entity[form/@type]"))
                        {
                            string strFormType = xnForm.SelectSingleNode("form/@type").InnerText;

                            //get the form events
                            foreach (XmlNode xnEvent in xnForm.SelectNodes("form/events/event"))
                            {
                                string strFormOrField = "form";
                                string strLabel = "";
                                string strMetaname = "";
                                string strEventName = xnEvent.SelectSingleNode("@name").InnerText;

                                string strScript = xnEvent.SelectSingleNode("script").InnerText;

                                myRange = thisApp.ActiveCell;
                                myRange.Value2 = strFormOrField;
                                myRange.get_Offset(0, 1).Value2 = strLabel;
                                myRange.get_Offset(0, 2).Value2 = strMetaname;
                                myRange.get_Offset(0, 3).Value2 = strEventName;

                                System.IO.StringReader sr = new System.IO.StringReader(strScript);
                                while (true)
                                {
                                    string strLine = sr.ReadLine();
                                    if (strLine != null)
                                    {
                                        thisApp.ActiveCell.get_Offset(0, 4).Value2 = strLine;
                                        thisApp.ActiveCell.get_Offset(1, 0).Activate();

                                    }
                                    else
                                        break;

                                }
                                //since this is the last row of the formula underline it.
                                thisApp.ActiveCell.get_Offset(-1, 0).EntireRow.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.get_Offset(-1, 0).EntireRow.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].Weight = Excel.XlBorderWeight.xlThin;
                            }

                            //get the field events
                            foreach (XmlNode xnEvent in xnForm.SelectNodes("form/tabs/tab/sections/section/rows/row/cell/events/event"))
                            {
                                string strFormOrField = "field";
                                string strLabel = "";
                                if (xnEvent.SelectSingleNode("../../labels/label/@description") != null)
                                    strLabel = xnEvent.SelectSingleNode("../../labels/label/@description").InnerText;
                                string strMetaName = "";
                                if (xnEvent.SelectSingleNode("../../control/@datafieldname")!= null)
                                    strMetaName = xnEvent.SelectSingleNode("../../control/@datafieldname").InnerText;
                                string strEventName = "";
                                if (xnEvent.SelectSingleNode("@name").InnerText != null)
                                    strEventName = xnEvent.SelectSingleNode("../../labels/label/@description").InnerText;
                                string strScript = "";
                                if (xnEvent.SelectSingleNode("script") != null);
                                    strScript = xnEvent.SelectSingleNode("script").InnerText;

                                myRange = thisApp.ActiveCell;
                                myRange.Value2 = strFormOrField;
                                myRange.get_Offset(0, 1).Value2 = strLabel;
                                myRange.get_Offset(0, 2).Value2 = strMetaName;
                                myRange.get_Offset(0, 3).Value2 = strEventName;

                                 //replace tabs with spaces (tabs are not displayed right in Excel)
                                System.Text.RegularExpressions.Regex MyRegex = new System.Text.RegularExpressions.Regex("\t");

                                System.IO.StringReader sr = new System.IO.StringReader(strScript);
                                while (true)
                                {
                                    string strLine = sr.ReadLine();

                                    if (strLine != null)
                                    {
                                       strLine = MyRegex.Replace(strLine, "   ");
                                        thisApp.ActiveCell.get_Offset(0, 4).Value2 = strLine;
                                        thisApp.ActiveCell.get_Offset(1, 0).Activate();
                                    }
                                    else
                                        break;
                                }
                                //since this is the last row of the formula underline it.
                                thisApp.ActiveCell.get_Offset(-1, 0).EntireRow.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                                thisApp.ActiveCell.get_Offset(-1, 0).EntireRow.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].Weight = Excel.XlBorderWeight.xlThin;
                            }
                        }
                        Excel.Worksheet tempworksheet = (Excel.Worksheet)thisApp.ActiveSheet;
                        Excel.Range myRange2;
                        // activate the first cell
                        myRange2 = tempworksheet.get_Range("A1", "A1");
                        myRange2.Activate();
                        // make columns fit
                        myRange2.EntireColumn.AutoFit();
                        myRange2 = tempworksheet.get_Range("B1", "B1");
                        myRange2.EntireColumn.AutoFit();
                        myRange2 = tempworksheet.get_Range("C1", "C1");
                        myRange2.EntireColumn.AutoFit();
                        // freeze/split first row
                        thisApp.ActiveWindow.SplitRow = 1;
                        thisApp.ActiveWindow.FreezePanes = true;
                        // make first row bold
                        myRange2.EntireRow.Font.Bold = true;
                    }
                }
                mySheet = (Excel.Worksheet) thisApp.Worksheets[mySettings.Entities[0]];
                mySheet.Activate();
            }
        }

        private void FormatColumnWidth(Excel.Application thisApp)
        {
            Excel.Worksheet mySheet = (Excel.Worksheet)thisApp.ActiveSheet;

            // activate the first cell
            Excel.Range myRange = mySheet.get_Range("A1", "XFD1048576");
            myRange.EntireColumn.AutoFit();
            myRange = mySheet.get_Range("A1", "A1");
            myRange.Activate();
        }


        private bool SheetExists(Excel.Application thisApp, string strName)
        {
            bool returnValue = false;
            try
            {

                foreach (Excel.Worksheet myWorksheet in thisApp.Worksheets)
                {
                    if (myWorksheet.Name == strName)
                    {
                        returnValue = true;
                    }
                }
            }
            catch (Exception ex)
            {
            }
            return returnValue;
        }

        private void CreateSheet(Excel.Application thisApp, string strName)
        {
            // create a sheet for this entity
            // TODO: if the sheet exists, delete and insert at the same sheet.....
            try
            {
                Excel.Worksheet myExistingWorksheet = null;
                // if it exists just skip it
                foreach (Excel.Worksheet myWorksheet in thisApp.Worksheets)
                {
                    if (myWorksheet.Name == strName)
                    {
                        myExistingWorksheet = myWorksheet;
                        myWorksheet.Activate();
                        //myWorksheet.Delete();
                        break;
                    }
                }
                if (myExistingWorksheet == null)
                {
                    Excel.Worksheet newWorksheet = (Excel.Worksheet)thisApp.ActiveWorkbook.Worksheets.Add(missing, thisApp.Worksheets[thisApp.Worksheets.Count], missing, missing);
                    newWorksheet.Name = strName;
                    newWorksheet.Activate();
                }
            }
            catch (Exception ex)
            {
            }
        }

        private void WriteFormRowHeader(Excel.Application thisApp)
        {
            //create the header
            Excel.Range myRange = thisApp.ActiveCell;

            myRange.Value2 = "Field";
            myRange.Font.Italic = true;
            myRange.Font.Size = 10;
            myRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
            myRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].Weight = Excel.XlBorderWeight.xlThin;
            myRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
            myRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;

            myRange.get_Offset(0, 1).Value2 = "Label on form";
            myRange.get_Offset(0, 1).Font.Italic = true;
            myRange.get_Offset(0, 1).Font.Size = 10;
            myRange.get_Offset(0, 1).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
            myRange.get_Offset(0, 1).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;

            myRange.get_Offset(0, 2).Value2 = "Label on Adv. Find";
            myRange.get_Offset(0, 2).Font.Italic = true;
            myRange.get_Offset(0, 2).Font.Size = 10;
            myRange.get_Offset(0, 2).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
            myRange.get_Offset(0, 2).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;

            myRange.get_Offset(0, 3).Value2 = "Type";
            myRange.get_Offset(0, 3).Font.Italic = true;
            myRange.get_Offset(0, 3).Font.Size = 10;
            myRange.get_Offset(0, 3).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
            myRange.get_Offset(0, 3).Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].Weight = Excel.XlBorderWeight.xlThin;

            myRange.get_Offset(1, 0).Activate();
        }

        private void WriteFormFormHeader(Excel.Application thisApp, string strEntityName, string strFormType)
        {
            try
            {
                //create the header 
                Excel.Range myRange = thisApp.ActiveCell;

                myRange.Value2 = "Entity: " + strEntityName + " (" + strFormType + " form)";
                myRange.Font.Size = 14;
                Excel.Range myMergeRange = thisApp.get_Range(thisApp.ActiveCell, thisApp.ActiveCell.get_Offset(0, 3));
                myMergeRange.Merge(missing);
                myRange.Font.Bold = true;

                myRange.Interior.Pattern = Excel.XlPattern.xlPatternSolid;
                myRange.Interior.PatternColorIndex = Excel.XlColorIndex.xlColorIndexAutomatic;
                myRange.Interior.ThemeColor = Excel.XlThemeColor.xlThemeColorLight1;
                myRange.Interior.TintAndShade = 0.599993896298105;
                myRange.Interior.PatternTintAndShade = 0;

                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].LineStyle = Excel.XlLineStyle.xlContinuous;
                myRange.get_Offset(1, 0).Activate();
            }
            catch (Exception ex)
            {
            }
        }

        private void WriteFormSectionHeader(Excel.Application thisApp, string strSectionName, string strSectionShowLabel)
        {
            try
            {
                //create the labels for the section
                Excel.Range myRange = thisApp.ActiveCell;
                myRange.Value2 = "Section: " + strSectionName;
                if (strSectionShowLabel == "false")
                    myRange.Value2 = (string)myRange.Value2 + " (hidden)";
                Excel.Range myMergeRange = thisApp.get_Range(thisApp.ActiveCell, thisApp.ActiveCell.get_Offset(0, 3));
                myMergeRange.Merge(missing);
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].LineStyle = Excel.XlLineStyle.xlContinuous;
                myRange.get_Offset(1, 0).Activate();
            }
            catch (Exception ex)
            {
            }
        }

        private void WriteFormTabHeader(Excel.Application thisApp, string strTabName)
        {
            try
            {
                // writes a row for a tab page found in CRM
                Excel.Range myRange = thisApp.ActiveCell;
                myRange.Value2 = "Tab: " + strTabName;
                myRange.Font.Bold = true;
                
                // merge cells
                Excel.Range myMergeRange = thisApp.get_Range(thisApp.ActiveCell, thisApp.ActiveCell.get_Offset(0, 3));
                myMergeRange.Merge(missing);

                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeRight].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeBottom].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeLeft].LineStyle = Excel.XlLineStyle.xlContinuous;
                myMergeRange.Borders[Microsoft.Office.Interop.Excel.XlBordersIndex.xlEdgeTop].LineStyle = Excel.XlLineStyle.xlContinuous;

                // when the tab is ready, switch to the next row
                myRange.get_Offset(1, 0).Activate();
            }
            catch (Exception ex)
            {
            }
        }
    }
}
