using System;
using System.Collections;
using System.Configuration.Install;
using System.Security.Policy;

namespace CRMDocumentationSetupHelper
{
    [System.ComponentModel.RunInstallerAttribute(true)]
    public class SecurityCustomAction : Installer
    {
        public SecurityCustomAction() : base()
        {
        }

        public override void Install(IDictionary stateSaver)
        {
            base.Install(stateSaver);

            string strPath = Context.Parameters["path"];
            string strName = Context.Parameters["name"];

            PolicyLevel plUserPolicyLevel = null;

            IEnumerator iePolicyLevels = System.Security.SecurityManager.PolicyHierarchy();
            while (iePolicyLevels.MoveNext())
            {
                PolicyLevel plPolicyLevel = (PolicyLevel)iePolicyLevels.Current;
                if (plPolicyLevel.Label == "User")
                    plUserPolicyLevel = plPolicyLevel;
            }

            if (plUserPolicyLevel != null)
            {
                UnionCodeGroup ucgRootCodeGroup = (UnionCodeGroup)plUserPolicyLevel.RootCodeGroup;
                System.Security.PermissionSet pmPermissionSet = plUserPolicyLevel.GetNamedPermissionSet("FullTrust");
                UrlMembershipCondition umcUrlMembershipCondition = new UrlMembershipCondition(strPath);
                UnionCodeGroup ucgNewUnionCodeGroup = new UnionCodeGroup(umcUrlMembershipCondition, new PolicyStatement(pmPermissionSet));
                ucgNewUnionCodeGroup.Name = strName;
                ucgRootCodeGroup.AddChild(ucgNewUnionCodeGroup);

                System.Security.SecurityManager.SavePolicy();
            }

        }

        public override void Uninstall(IDictionary savedState)
        {
            base.Uninstall(savedState);

            string strPath = Context.Parameters["path"];
            string strName = Context.Parameters["name"];

            PolicyLevel plUserPolicyLevel = null;

            IEnumerator iePolicyLevels = System.Security.SecurityManager.PolicyHierarchy();
            while (iePolicyLevels.MoveNext())
            {
                PolicyLevel plPolicyLevel = (PolicyLevel)iePolicyLevels.Current;
                if (plPolicyLevel.Label == "User")
                    plUserPolicyLevel = plPolicyLevel;
            }

            if (plUserPolicyLevel != null)
            {
                UnionCodeGroup ucgRootCodeGroup = (UnionCodeGroup)plUserPolicyLevel.RootCodeGroup;
                string strResolvedPath = new UrlMembershipCondition(strPath).ToString();

                foreach (CodeGroup cgCodeGroup in ucgRootCodeGroup.Children)
                {
                    if (cgCodeGroup.Name == strName)
                        if (cgCodeGroup.MembershipCondition.ToString() == strResolvedPath && cgCodeGroup.PermissionSetName == "FullTrust")
                        {
                            ucgRootCodeGroup.RemoveChild(cgCodeGroup);
                            System.Security.SecurityManager.SavePolicy();
                            break;
                        }
                }
            }
        }
    }
}